@@ -36,7 +36,7 @@
 #include "var.h"
 
 /* this mutex provides a lock for stdout to avoid display conflict between thread */
-G_LOCK_DEFINE_STATIC(stdout);
+G_LOCK_DEFINE_STATIC(std_out);
 
 static FILE *log_fd=NULL;
 static FILE *errlog_fd=NULL;
@@ -104,6 +104,7 @@ static const char *msg_str_type[]={
 										"DRLAY]",		/* this message is never displayed, only the message content is */
 										"FLSTB]",
 										"LUSER]",
+										"LSCCH]",
                               };
 
 #define ERR_TO_LOG(x)	if((errlog_fd!=NULL)&&(msg_type==ERR_MSG))	{x;}
@@ -134,7 +135,7 @@ void disp_msg_full(DISP_MSG_TYPE msg_type,GString **produced_string,const char *
 	if((msg_type==DEBUG_MSG)&&(debug_mode==0))
 		return;
 
-	G_LOCK(stdout);
+	G_LOCK(std_out);
 	o_str=g_string_new("");
 
 	va_start(ap,fnc_name);
@@ -257,7 +258,7 @@ void disp_msg_full(DISP_MSG_TYPE msg_type,GString **produced_string,const char *
 	va_end(ap);
 
 	fflush(stdout);			/* flush stdout before unlocking */
-	G_UNLOCK(stdout);
+	G_UNLOCK(std_out);
 }
 
 /***********************************/
@@ -268,7 +269,7 @@ void disp_msg_full(DISP_MSG_TYPE msg_type,GString **produced_string,const char *
 void change_logfile(char *filename)
 {
 	/* to avoid potential access conflict, we lock the display */
-	G_LOCK(stdout);
+	G_LOCK(std_out);
 
 	/* close the previously existing FILE */
 	if(log_fd!=NULL)
@@ -286,7 +287,7 @@ void change_logfile(char *filename)
 			disp_msg(ERR_MSG,"change_logfile",strerror(errno),NULL);
 		}
 	}
-	G_UNLOCK(stdout);
+	G_UNLOCK(std_out);
 }
 
 /***************************************/
@@ -297,7 +298,7 @@ void change_logfile(char *filename)
 void change_errlogfile(char *filename)
 {
 	/* to avoid potential access conflict, we lock the display */
-	G_LOCK(stdout);
+	G_LOCK(std_out);
 
 	/* close the previously existing FILE */
 	if(errlog_fd!=NULL)
@@ -315,6 +316,6 @@ void change_errlogfile(char *filename)
 			disp_msg(ERR_MSG,"change_errlogfile",strerror(errno),NULL);
 		}
 	}
-	G_UNLOCK(stdout);
+	G_UNLOCK(std_out);
 }
 
