@@ -34,6 +34,7 @@
 #include <fcntl.h>
 #include <pthread.h>
 #include <sys/socket.h>
+#include <sys/param.h>
 #include <ctype.h>
 #include <netinet/in.h>
 #include <glib.h>
@@ -139,7 +140,8 @@ static int gdldle_index(GDL_ENTRY *gdle, char *nickname, char *remote_fname)
 	return -1;
 }
 
-typedef struct {unsigned long b; unsigned long int e;} S_RNG;
+//typedef struct {unsigned long b; unsigned long int e;} S_RNG;
+typedef struct {off_t b; off_t e;} S_RNG;
 
 static int com_s_rng(const void *a, const void *b)
 {
@@ -499,9 +501,64 @@ static int allocate_range(GDL_DL_ENTRY *gdldle, GDL_ENTRY *gdle)
 	return 1;	/* range allocated */
 }
 
+/********************************************************************************************/
+/* check if the number of active connection with a user does not exceed the maximum allowed */
+/* Note: started GDL sources are taken into account as well as running ones                 */
+/* NOTE: gdl_array is already locked when entering in this function                         */
+/********************************************************************************************/
+/* output: 0= limit not reached, 1= limit reached */
+/**************************************************/
+static int enough_dl_connection_with_user(char *nickname, int max_cnx)
+{
+	int ret=0;
+
+	if(max_cnx>=1)
+	{
+		int j=0;
+		while(j<gdl_array->len)
+		{
+			const GDL_ENTRY *gdle;
+			gdle=g_ptr_array_index(gdl_array,j);
+
+			if(gdle!=NULL)
+			{
+				const GPtrArray *gdl_links;
+
+				gdl_links=gdle->gdl_links;
+				if(gdl_links!=NULL)
+				{
+					int i;
+					for(i=0;i<gdl_links->len;i++)
+					{
+						GDL_DL_ENTRY *gdldle;
+	
+						gdldle=g_ptr_array_index(gdl_links,i);
+
+						if((gdldle!=NULL)&&(gdldle->is_running)&&(!strcmp(nickname,gdldle->nickname->str)))
+						{
+							max_cnx--;
+							if(max_cnx<=0)
+							{
+								ret=1;
+								goto abrt;
+							}
+						}
+					}
+				}
+			}
+			j++;
+		}
+	}
+
+	abrt:
+	return ret;
+}
+
 /*****************************/
 /* try to start its download */
-/*****************************/
+/********************************************************************/
+/* NOTE: gdl_array is already locked when entering in this function */
+/********************************************************************/
 static void start_xdl(GDL_ENTRY *gdle, GDL_DL_ENTRY *gdldle, unsigned long int min_position)
 {
 	GString *nw;
@@ -511,6 +568,9 @@ static void start_xdl(GDL_ENTRY *gdle, GDL_DL_ENTRY *gdldle, unsigned long int m
 	if(gdldle->remote_fsize<=min_position)	/* the file of this source has no free range */
 		return;
 
+	if(enough_dl_connection_with_user(gdldle->nickname->str,max_dl_per_user))
+		return;
+
 	if(!user_in_list(hub_user_list,gdldle->nickname->str))		/* user on the hub */
 	{
 		if((with_ddl==0)||(!check_uaddr_entry_by_name(gdldle->nickname->str)))	/* or DDL enable and user address is known */
@@ -556,6 +616,8 @@ static int has_running_not_started_xdl(GDL_ENTRY *gdle, GDL_DL_ENTRY *gdldle)
 /*********************************************************************/
 /* scan the given ENTRY and try to start downloads on inactive links */
 /*********************************************************************/
+/* NOTE: gdl_array is already locked when entering in this function */
+/********************************************************************/
 static void handle_one_gdl(GDL_ENTRY *gdle)
 {
 	int i;
@@ -700,6 +762,102 @@ static int get_range_entry_for_position(GArray *dld_ranges, unsigned long cur_po
 	return -1;
 }
 
+static int recursive_build_directory(GString *dir)
+{
+	struct stat st;
+
+	if(stat(dir->str,&st)==0)
+	{
+		return 0;
+	}
+	else
+	{
+		GString *new_dir;
+		char *t;
+		int ret;
+
+		new_dir=g_string_new(dir->str);
+		t=strrchr(new_dir->str,'/');
+		if(t==NULL)
+		{
+			g_string_free(new_dir,TRUE);
+			return ENOTDIR;
+		}
+
+		new_dir=g_string_truncate(new_dir,t-new_dir->str);
+		ret=recursive_build_directory(new_dir);
+		g_string_free(new_dir,TRUE);
+
+		if(ret!=0)
+			return ret;
+
+		if(mkdir(dir->str,0777))
+		{
+			return errno;
+		}
+		return 0;
+	}
+}
+
+/******************************************************/
+/* create the given directory (it can start with a /) */
+/******************************************************/
+/* output: 0=ok else errno */
+/***************************/
+static int recursive_mkdir(GString *directory_name)
+{
+	GString *final_name;
+	int ret;
+
+	if(directory_name->str[0]=='/')
+		final_name=g_string_new(directory_name->str);
+	else
+	{
+		char the_cwd[MAXPATHLEN];
+		if(getcwd(the_cwd,sizeof(the_cwd)-1)==NULL)
+		{
+			return errno;
+		}
+
+		final_name=g_string_new(the_cwd);
+		if(final_name->str[final_name->len-1]!='/')
+			final_name=g_string_append_c(final_name,'/');
+		final_name=g_string_append(final_name,directory_name->str);
+	}
+
+	/* now, final_name is an absolute path */
+	ret=recursive_build_directory(final_name);
+
+	g_string_free(final_name,TRUE);
+
+	return ret;
+}
+
+static void start_at_end_script(char *programname, char *gdl_filename)
+{
+	switch(fork())
+	{
+		case -1:	disp_msg(ERR_MSG,"start_at_end_script","fork fails",NULL);
+					break;
+
+		case 0:	/* the son */
+					{	/* close all files descriptors */
+						int i;
+						int mx;
+
+						mx=sysconf(_SC_OPEN_MAX);
+							for(i=2;i<mx;i++)
+								close(i);
+					}
+
+					/* and start the program */
+					execlp(programname,programname,gdl_filename,NULL);
+					_exit(0);
+
+		default:	return;
+	}
+}
+
 /*********************************************/
 /* gather part of the file into one big file */
 /*********************************************/
@@ -707,42 +865,62 @@ static int get_range_entry_for_position(GArray *dld_ranges, unsigned long cur_po
 /*******************************/
 static int gather_gdl_part(GDL_ENTRY *gdle)
 {
-	FILE *f;
-	unsigned long cur_pos=0;
-	int idx;
 	int a;
+	int copy_done=0;
+	GString *terminal_filename=NULL;
 
-	f=fopen(gdle->local_fname->str,"wb");
-	if(f==NULL)
+	if(gdle->dld_ranges->len==1)
 	{
-		a=errno;
-		disp_msg(ERR_MSG,"gather_gdl_part","unable to create",gdle->local_fname->str,strerror(a),NULL);
-		return 1;		/* error */
+		/* 1 segment only, try to avoid the copy */
+		RANGE_ENTRY *ptr=&(g_array_index(gdle->dld_ranges,RANGE_ENTRY,0));
+		a=rename(ptr->temp_local_fname->str,gdle->local_fname->str);
+
+		if(a==0)
+			copy_done=1;
+		else
+		{
+			disp_msg(ERR_MSG,"gather_gdl_part","mono-fragment file rename fail",ptr->temp_local_fname->str,"To",gdle->local_fname->str,strerror(a),"Trying in copy mode",NULL);
+		}
 	}
 
-	cur_pos=0;
-	while(cur_pos!=gdle->total_size)
+	if(!copy_done)
 	{
-		idx=get_range_entry_for_position(gdle->dld_ranges,cur_pos);
-		if(idx==-1)
+		FILE *f;
+		unsigned long cur_pos=0;
+		int idx;
+
+		f=fopen(gdle->local_fname->str,"wb");
+		if(f==NULL)
 		{
-			disp_msg(ERR_MSG,"gather_gdl_part","Internal error, a completed GDL contains missing parts",gdle->local_fname->str,
-									"position","|lu",(unsigned long)cur_pos,NULL);
-			fclose(f);
-			unlink(gdle->local_fname->str);
+			a=errno;
+			disp_msg(ERR_MSG,"gather_gdl_part","unable to create",gdle->local_fname->str,strerror(a),NULL);
 			return 1;		/* error */
 		}
 
-		if((a=append_this_range_entry_to_file(f,&cur_pos,&(g_array_index(gdle->dld_ranges,RANGE_ENTRY,idx))))!=0)
+		cur_pos=0;
+		while(cur_pos!=gdle->total_size)
 		{
-			disp_msg(ERR_MSG,"gather_gdl_part","Fail to append",(g_array_index(gdle->dld_ranges,RANGE_ENTRY,idx)).temp_local_fname->str,"at the end of",
-									gdle->local_fname->str,"Reason:",strerror(a),NULL);
-			fclose(f);
-			unlink(gdle->local_fname->str);
-			return 1;		/* error */
+			idx=get_range_entry_for_position(gdle->dld_ranges,cur_pos);
+			if(idx==-1)
+			{
+				disp_msg(ERR_MSG,"gather_gdl_part","Internal error, a completed GDL contains missing parts",gdle->local_fname->str,
+										"position","|lu",(unsigned long)cur_pos,NULL);
+				fclose(f);
+				unlink(gdle->local_fname->str);
+				return 1;		/* error */
+			}
+	
+			if((a=append_this_range_entry_to_file(f,&cur_pos,&(g_array_index(gdle->dld_ranges,RANGE_ENTRY,idx))))!=0)
+			{
+				disp_msg(ERR_MSG,"gather_gdl_part","Fail to append",(g_array_index(gdle->dld_ranges,RANGE_ENTRY,idx)).temp_local_fname->str,"at the end of",
+										gdle->local_fname->str,"Reason:",strerror(a),NULL);
+				fclose(f);
+				unlink(gdle->local_fname->str);
+				return 1;		/* error */
+			}
 		}
+		fclose(f);
 	}
-	fclose(f);
 
 	{
 		FILE *f;
@@ -765,42 +943,255 @@ static int gather_gdl_part(GDL_ENTRY *gdle)
 		g_string_free(o,TRUE);
 	}
 
-	if(when_done)
+
+	/* here, we have 6 cases: */
+	/* when_done flag | post_fname | post_dir */
+	/*  not set       |  NULL      | NULL     : 1) (v=0) the easiest case, nothing to do */
+	/*  set           |  NULL      | NULL     : 2) (v=1) move the created file into the done directory */
+	/*  not set       |  not NULL  | NULL     : 3) (v=2) rename the created file to its new name */
+	/*  set           |  not NULL  | NULL     : 4) (v=3) move the created file into the done directory and change its new name */
+	/*  not set       |  not NULL  | not NULL : 5) (v=6) change the name of the created file and put it in a different directory */
+	/*  set           |  not NULL  | not NULL : 6) (v=7) change the name of the created file and put it in a different directory located in the done directory */
+
 	{
-		GString *dir_part;
-		char *t;
-		int offset;
+		int post_fname_flag, post_dir_flag;
+		int global_check;
 
-		/* source: gdle->local_fname->str */
-
-		dir_part=g_string_new(gdle->local_fname->str);
-		t=strrchr(dir_part->str,'/');
-		if(t!=NULL)
+		if((gdle->post_fname==NULL)||(gdle->post_fname->len==0))
 		{
-			offset=t-dir_part->str+1;	  /* position just after the / */
+			post_fname_flag=0;
+			post_dir_flag=0;
 		}
 		else
 		{
-			offset=0;
+			post_fname_flag=1;
+			if((gdle->post_dir==NULL)||(gdle->post_dir->len==0))
+				post_dir_flag=0;
+			else
+				post_dir_flag=1;
 		}
-		dir_part=g_string_truncate(dir_part,offset);
-		dir_part=g_string_append(dir_part,"done");
 
-		/* create the destination directory */
-		mkdir(dir_part->str,0777);
+		global_check=(when_done?1:0)|(post_fname_flag<<1)|(post_dir_flag<<2);
 
-		dir_part=g_string_assign(dir_part,gdle->local_fname->str);
-		dir_part=g_string_insert(dir_part,offset,"done/");
-		/* move the file */
-		if(rename(gdle->local_fname->str,dir_part->str))
+		/* default filename at the end */
+		terminal_filename=g_string_new(gdle->local_fname->str);
+
+		switch(global_check)
 		{
-			int err=errno;
+			case 0:	/* case 1: nothing */
+						break;
 
-			disp_msg(ERR_MSG,NULL,"Fail to rename",gdle->local_fname->str,"into",dir_part->str,"because:",strerror(err),NULL);
+			case 1:	/* case 2: when_done */
+						{
+							GString *dir_part;
+							char *t;
+							int offset;
+
+							/* source: gdle->local_fname->str */
+
+							dir_part=g_string_new(gdle->local_fname->str);
+							t=strrchr(dir_part->str,'/');
+							if(t!=NULL)
+							{
+								offset=t-dir_part->str+1;	  /* position just after the / */
+							}
+							else
+							{
+								offset=0;
+							}
+							dir_part=g_string_truncate(dir_part,offset);
+							dir_part=g_string_append(dir_part,"done");
+
+							/* create the destination directory */
+							mkdir(dir_part->str,0777);
+					
+							dir_part=g_string_assign(dir_part,gdle->local_fname->str);
+							dir_part=g_string_insert(dir_part,offset,"done/");
+							/* move the file */
+							if(rename(gdle->local_fname->str,dir_part->str))
+							{
+								int err=errno;
+					
+								disp_msg(ERR_MSG,NULL,"Fail to rename (1)",gdle->local_fname->str,"into",dir_part->str,"because:",strerror(err),NULL);
+							}
+							else
+								terminal_filename=g_string_assign(terminal_filename,dir_part->str);
+							g_string_free(dir_part,TRUE);
+						}
+						break;
+
+			case 2:	/* case 3: post_fname */
+						{
+							GString *dir_part;
+							char *t;
+							int offset;
+
+							/* source: gdle->local_fname->str */
+
+							dir_part=g_string_new(gdle->local_fname->str);
+							t=strrchr(dir_part->str,'/');
+							if(t!=NULL)
+							{
+								offset=t-dir_part->str+1;	  /* position just after the / */
+							}
+							else
+							{
+								offset=0;
+							}
+							dir_part=g_string_truncate(dir_part,offset);
+							dir_part=g_string_append(dir_part,gdle->post_fname->str);
+
+							/* move the file */
+							if(rename(gdle->local_fname->str,dir_part->str))
+							{
+								int err=errno;
+					
+								disp_msg(ERR_MSG,NULL,"Fail to rename (2)",gdle->local_fname->str,"into",dir_part->str,"because:",strerror(err),NULL);
+							}
+							else
+								terminal_filename=g_string_assign(terminal_filename,dir_part->str);
+							g_string_free(dir_part,TRUE);
+						}
+						break;
+
+			case 3:	/* case 4: when_done+post_fname */
+						{
+							GString *dir_part;
+							char *t;
+							int offset;
+
+							/* source: gdle->local_fname->str */
+
+							dir_part=g_string_new(gdle->local_fname->str);
+							t=strrchr(dir_part->str,'/');
+							if(t!=NULL)
+							{
+								offset=t-dir_part->str+1;	  /* position just after the / */
+							}
+							else
+							{
+								offset=0;
+							}
+							dir_part=g_string_truncate(dir_part,offset);
+							dir_part=g_string_append(dir_part,"done");
+
+							/* create the destination directory */
+							mkdir(dir_part->str,0777);
+					
+							dir_part=g_string_append_c(dir_part,'/');
+							dir_part=g_string_append(dir_part,gdle->post_fname->str);
+							/* move the file */
+							if(rename(gdle->local_fname->str,dir_part->str))
+							{
+								int err=errno;
+					
+								disp_msg(ERR_MSG,NULL,"Fail to rename (3)",gdle->local_fname->str,"into",dir_part->str,"because:",strerror(err),NULL);
+							}
+							else
+								terminal_filename=g_string_assign(terminal_filename,dir_part->str);
+							g_string_free(dir_part,TRUE);
+						}
+						break;
+
+			case 6:	/* case 5: post_fname+post_dir */
+						{
+							GString *dir_part;
+							char *t;
+							int offset;
+							int err;
+
+							/* source: gdle->local_fname->str */
+
+							dir_part=g_string_new(gdle->local_fname->str);
+							t=strrchr(dir_part->str,'/');
+							if(t!=NULL)
+							{
+								offset=t-dir_part->str+1;	  /* position just after the / */
+							}
+							else
+							{
+								offset=0;
+							}
+							dir_part=g_string_truncate(dir_part,offset);
+							dir_part=g_string_append(dir_part,gdle->post_dir->str);
+							if((err=recursive_mkdir(dir_part))==0)
+							{
+								dir_part=g_string_append_c(dir_part,'/');
+								dir_part=g_string_append(dir_part,gdle->post_fname->str);
+
+								/* move the file */
+								if(rename(gdle->local_fname->str,dir_part->str))
+								{
+									err=errno;
+					
+									disp_msg(ERR_MSG,NULL,"Fail to rename (6)",gdle->local_fname->str,"into",dir_part->str,"because:",strerror(err),NULL);
+								}
+								else
+									terminal_filename=g_string_assign(terminal_filename,dir_part->str);
+							}
+							else
+							{
+								disp_msg(ERR_MSG,NULL,"recursive_mkdir fails for (6)",dir_part->str,"because:",strerror(err),NULL);
+							}
+							g_string_free(dir_part,TRUE);
+						}
+						break;
+
+			case 7:	/* case 6: when_done+post_fname+post_dir */
+						{
+							GString *dir_part;
+							char *t;
+							int offset;
+							int err;
+
+							/* source: gdle->local_fname->str */
+
+							dir_part=g_string_new(gdle->local_fname->str);
+							t=strrchr(dir_part->str,'/');
+							if(t!=NULL)
+							{
+								offset=t-dir_part->str+1;	  /* position just after the / */
+							}
+							else
+							{
+								offset=0;
+							}
+							dir_part=g_string_truncate(dir_part,offset);
+							dir_part=g_string_append(dir_part,"done/");
+							dir_part=g_string_append(dir_part,gdle->post_dir->str);
+							if((err=recursive_mkdir(dir_part))==0)
+							{
+								dir_part=g_string_append_c(dir_part,'/');
+								dir_part=g_string_append(dir_part,gdle->post_fname->str);
+
+								/* move the file */
+								if(rename(gdle->local_fname->str,dir_part->str))
+								{
+									err=errno;
+					
+									disp_msg(ERR_MSG,NULL,"Fail to rename (7)",gdle->local_fname->str,"into",dir_part->str,"because:",strerror(err),NULL);
+								}
+								else
+									terminal_filename=g_string_assign(terminal_filename,dir_part->str);
+							}
+							else
+							{
+								disp_msg(ERR_MSG,NULL,"recursive_mkdir fails for (7)",dir_part->str,"because:",strerror(err),NULL);
+							}
+							g_string_free(dir_part,TRUE);
+						}
+						break;
 		}
-		g_string_free(dir_part,TRUE);
 	}
 
+	if(terminal_filename!=NULL)
+	{
+		if(gdle->at_end_script!=NULL)
+		{
+			start_at_end_script(gdle->at_end_script->str,terminal_filename->str);
+		}
+		g_string_free(terminal_filename,TRUE);
+	}
 	return 0;
 }
 
@@ -887,11 +1278,13 @@ static void do_gdl_autoscan(GDL_ENTRY *gdle)
 			GString *str;
 			str=g_string_new("");
 
+			LOCK_READ(user_info);
 			g_string_sprintf(str,"$Search %s:%hu T?F?%lu?%s|",														/* the size is always important, it is at least */
 												host_ip,gdle->autoscan_sock_port,
 												gdle->total_size-1,			/* it is not possible to ask for an exact size (N) */
 																					/* to reduce bandwidth waste, we ask a size of at least N-1 bytes */
 												gae->search_pattern);		/* it is a datatype?pattern in fact */
+			UNLOCK_READ(user_info);
 			
 			if((main_sck!=-1)&&(cnx_in_progress==0))
 			{	/* if connected to the hub, send the message to this hub */
@@ -997,7 +1390,7 @@ static void	receive_and_decode_autoscan_result(GDL_ENTRY *gdle)
 						gdl_wake_up_sources(nick,1);		/* don't lock again or this will hang */
 					}
 				}
-   		}
+	  		}
 		}
 	}
 }
@@ -1127,8 +1520,8 @@ static void gdl_thread(void *dummy)
 /**********************************************************/
 void start_gdl_thread(void)
 {
-	pthread_attr_t thread_attr;
-	pthread_t thread_id;
+	static pthread_attr_t thread_attr;	/* these 2 variables must exist as long as the thread exist */
+	static pthread_t thread_id;
 
 	if(gdl_array==NULL)
 	{
@@ -1259,6 +1652,29 @@ static void update_gdl_cmd_of_this_gdle(GDL_ENTRY *gdle)
 	}
 	else
 	{
+		/* add file renaming if exists */
+		if((gdle->post_fname!=NULL)&&(strlen(gdle->post_fname->str)))
+		{
+			g_string_sprintf(str,"O|%s|%s|-\n",gdle->post_fname->str,
+															(gdle->post_dir!=NULL)?gdle->post_dir->str:"");
+			if(fwrite(str->str,1,str->len,f)!=str->len)
+			{
+				disp_msg(ERR_MSG,"update_gdl_cmd_of_this_gdle","write fail",str->str,NULL);
+				goto abrt;
+			}
+		}
+	
+		/* script to start at end if exists */
+		if((gdle->at_end_script!=NULL)&&(strlen(gdle->at_end_script->str)))
+		{
+			g_string_sprintf(str,"P|%s|-|-\n",gdle->at_end_script->str);
+			if(fwrite(str->str,1,str->len,f)!=str->len)
+			{
+				disp_msg(ERR_MSG,"update_gdl_cmd_of_this_gdle","write fail",str->str,NULL);
+				goto abrt;
+			}
+		}
+	
 		/* put the GDL source entries */
 		for(i=0;i<gdle->gdl_links->len;i++)
 		{
@@ -1310,6 +1726,7 @@ static void update_gdl_cmd_of_this_gdle(GDL_ENTRY *gdle)
 				goto abrt;
 			}
 		}
+
 	}
 
 	abrt:
@@ -1357,6 +1774,9 @@ int do_gdl_new(unsigned int gdl_id, char *local_filename, unsigned long int tota
 			nw->start_time=time(NULL);
 			nw->is_completed=0;
 			nw->local_fname=g_string_new(local_filename);
+			nw->post_fname=NULL;
+			nw->post_dir=NULL;
+			nw->at_end_script=NULL;
 			nw->total_size=total_size;
 			nw->gdl_links=g_ptr_array_new();
 			nw->dld_ranges=g_array_new(FALSE,FALSE,sizeof(RANGE_ENTRY));
@@ -1382,6 +1802,30 @@ int do_gdl_new(unsigned int gdl_id, char *local_filename, unsigned long int tota
 	return ret;
 }
 
+/************************/
+/* .cmd file line type: 
+- The main line, must be the first
+G|gdl_id|local filename|local filesize
+
+- download source
+S|nickname|remote filename|remote filesize
+
+- fragment having download in progress
+A|local fragment name|fragment start position|-
+
+- downloaded fragment
+R|local fragment name|fragment start position|fragment end position
+
+- search pattern
+F|pattern|-|-
+
+- local file renaming
+O|local filename|local dirname|-
+
+- program to start at end of GDL
+P|local program name|-|-
+
+*/
 /*****************************************************/
 /* load the .cmd file into the given empty GDL entry */
 /*****************************************************/
@@ -1435,6 +1879,9 @@ static int load_gdl_cmd(GDL_ENTRY *gdle, char *filename)
 		goto abrt;
 	
 	gdle->local_fname=g_string_new(fields[2]);
+	gdle->post_fname=NULL;
+	gdle->post_dir=NULL;
+	gdle->at_end_script=NULL;
 	gdle->total_size=strtoul(fields[3],NULL,10);
 	g_strfreev(fields);
 
@@ -1494,6 +1941,31 @@ static int load_gdl_cmd(GDL_ENTRY *gdle, char *filename)
 				gdle->dl_offset+=(re.range[1]-re.range[0]);
 			}
 		}
+		else if(!strcmp(fields[0],"O"))
+		{
+			if(gdle->post_fname)
+			{
+				g_string_free(gdle->post_fname,TRUE);
+				gdle->post_fname=NULL;
+			}
+
+			if(gdle->post_dir)
+			{
+				g_string_free(gdle->post_dir,TRUE);
+				gdle->post_dir=NULL;
+			}
+
+			if(strlen(fields[1]))
+			{
+				gdle->post_fname=g_string_new(fields[1]);
+
+				if(strlen(fields[2]))
+				{
+					gdle->post_dir=g_string_new(fields[2]);
+				}
+			}
+			
+		}
 		else if(!strcmp(fields[0],"F"))
 		{
 			GDL_AS_ENTRY gae;
@@ -1505,7 +1977,7 @@ static int load_gdl_cmd(GDL_ENTRY *gdle, char *filename)
 
 			if(gdle->autoscan_sockfd==-1)
 			{
-				gdle->autoscan_sockfd=_x_udp(0);		/* create an UDP socket and found an unused port */
+				gdle->autoscan_sockfd=_x_udp(gdl_as_port_range[0],gdl_as_port_range[1]);		/* create an UDP socket and found an unused port */
 				if(gdle->autoscan_sockfd==-1)
 				{
 					disp_msg(ERR_MSG,"load_gdl_cmd","WARNING: unable to create UDP socket for autoscan of this GDL","|lu",(unsigned long)(gdle->gdl_id),NULL);
@@ -1517,6 +1989,7 @@ static int load_gdl_cmd(GDL_ENTRY *gdle, char *filename)
 					int len_lcl=sizeof(lcl);
 
 					set_non_bloquant_sock(gdle->autoscan_sockfd);
+					set_tos_sock(gdle->autoscan_sockfd,udp_tos);
 
 					if(getsockname(gdle->autoscan_sockfd,(void*)&lcl,&len_lcl)!=0)
 					{
@@ -1532,6 +2005,12 @@ static int load_gdl_cmd(GDL_ENTRY *gdle, char *filename)
 				}
 			}
 		}
+		else if(!strcmp(fields[0],"P"))
+		{
+			if(gdle->at_end_script!=NULL)
+				g_string_free(gdle->at_end_script,TRUE);
+			gdle->at_end_script=g_string_new(fields[1]);
+		}
 		else
 				goto abrt;
 		g_strfreev(fields);
@@ -1745,6 +2224,7 @@ static void recur_del(GString *str)
 	}
 
 	g_string_free(name,TRUE);
+	closedir(dir);
 	rmdir(str->str);
 }
 
@@ -2078,6 +2558,13 @@ int do_gdl_end(unsigned int gdl_id,int override)
 		g_string_free(gdle->local_fname,TRUE);
 	}
 
+	if(gdle->post_fname)
+		g_string_free(gdle->post_fname,TRUE);
+	if(gdle->post_dir)
+		g_string_free(gdle->post_dir,TRUE);
+	if(gdle->at_end_script)
+		g_string_free(gdle->at_end_script,TRUE);
+
 	free(gdle);
 
 	return 0;
@@ -2116,7 +2603,7 @@ void do_gdl_as_add(unsigned int gdl_id, int filetype, char *pattern)
 
 		if(gdle->autoscan_sockfd==-1)
 		{
-			gdle->autoscan_sockfd=_x_udp(0);		/* create an UDP socket and found an unused port */
+			gdle->autoscan_sockfd=_x_udp(gdl_as_port_range[0],gdl_as_port_range[1]);		/* create an UDP socket and found an unused port */
 			if(gdle->autoscan_sockfd==-1)
 			{
 				disp_msg(ERR_MSG,"do_gdl_as_add","WARNING: unable to create UDP socket for autoscan of this GDL","|lu",(unsigned long)(gdle->gdl_id),NULL);
@@ -2128,6 +2615,7 @@ void do_gdl_as_add(unsigned int gdl_id, int filetype, char *pattern)
 				int len_lcl=sizeof(lcl);
 
 				set_non_bloquant_sock(gdle->autoscan_sockfd);
+				set_tos_sock(gdle->autoscan_sockfd,udp_tos);
 
 				if(getsockname(gdle->autoscan_sockfd,(void*)&lcl,&len_lcl)!=0)
 				{
@@ -2201,6 +2689,128 @@ void do_gdl_as_del(unsigned int gdl_id, unsigned long gae_id)
 	return;
 }
 
+/****************************************************/
+/* change the final filename [and dirname] of a GDL */
+/*********************************************************************/
+/* if dirname == "", only filename is altered                        */
+/* if both filename and dirname == "", the file renaming is disabled */
+/*********************************************************************/
+void do_gdl_rename(unsigned int gdl_id, char *filename, char *dirname)
+{
+	int idx;
+	G_LOCK(gdl_array);
+
+	/* first, check if the given index is not already used */
+	idx=gdl_index(gdl_id);
+	if(idx!=-1)
+	{	
+		GDL_ENTRY *gdle;
+
+		gdle=g_ptr_array_index(gdl_array,idx);
+
+		if((filename==NULL)||(strlen(filename)==0))
+		{
+			/* no filename -> no rename */
+			if(gdle->post_fname!=NULL)
+			{
+				g_string_free(gdle->post_fname,TRUE);
+				gdle->post_fname=NULL;
+			}
+			if(gdle->post_dir!=NULL)
+			{
+				g_string_free(gdle->post_dir,TRUE);
+				gdle->post_dir=NULL;
+			}
+		}
+		else
+		{
+			/* a filename is given */
+			if(gdle->post_fname!=NULL)
+				gdle->post_fname=g_string_assign(gdle->post_fname,filename);
+			else
+				gdle->post_fname=g_string_new(filename);
+
+			if(gdle->post_dir==NULL)
+				gdle->post_dir=g_string_new("");
+			
+			if(dirname!=NULL)
+			{
+				gdle->post_dir=g_string_assign(gdle->post_dir,dirname);
+			}
+		}
+		update_gdl_cmd_of_this_gdle(gdle);		/* update the cmd */
+	}
+	
+	G_UNLOCK(gdl_array);
+	return;
+}
+
+/***************************************************************/
+/* change the name of the program to start at the end of a GDL */
+/***************************************************************/
+/* if filename is NULL or == "", the file renaming is disabled */
+/***************************************************************/
+void do_gdl_script(unsigned int gdl_id, char *programname)
+{
+	int idx;
+	G_LOCK(gdl_array);
+
+	/* first, check if the given index is not already used */
+	idx=gdl_index(gdl_id);
+	if(idx!=-1)
+	{	
+		GDL_ENTRY *gdle;
+
+		gdle=g_ptr_array_index(gdl_array,idx);
+
+		if((programname==NULL)||(strlen(programname)==0))
+		{
+			/* no programname -> no script */
+			if(gdle->at_end_script!=NULL)
+			{
+				g_string_free(gdle->at_end_script,TRUE);
+				gdle->at_end_script=NULL;
+			}
+		}
+		else
+		{
+			/* a programname is given */
+			if(gdle->at_end_script!=NULL)
+				gdle->at_end_script=g_string_assign(gdle->at_end_script,programname);
+			else
+				gdle->at_end_script=g_string_new(programname);
+		}
+		update_gdl_cmd_of_this_gdle(gdle);		/* update the cmd */
+	}
+	
+	G_UNLOCK(gdl_array);
+	return;
+}
+/************************************************************/
+/* call this function to update already existing socket TOS */
+/************************************************************/
+void gdl_alter_socket_tos(void)
+{
+	int i;
+
+	G_LOCK(gdl_array);
+	if(gdl_array!=NULL)
+	{
+		for(i=0;i<gdl_array->len;i++)
+		{
+			GDL_ENTRY *gdle;
+
+			gdle=g_ptr_array_index(gdl_array,i);
+
+			if(gdle->autoscan_sockfd!=-1)
+				set_tos_sock(gdle->autoscan_sockfd,udp_tos);
+		}
+	}
+	G_UNLOCK(gdl_array);
+	return;
+}
+
+
 /* -------------------------------------------------------------------------- */
 /*************************************************************************/
 /* the following functions are used by the standard download function to */
@@ -2889,6 +3499,9 @@ void do_gdl_lst(int sck)
 													"|lu",(unsigned long)(dld_size),					/* amount of bytes still here */
 													"|lu",(unsigned long)(gdle->start_time),		/* creation/attachement time */
 													"|lu",(unsigned long)(gdle->cur_spd),			/* number of bytes received in the previous 10 seconds range */
+													"|s",(gdle->post_dir)?gdle->post_dir->str:"",		/* directory rename */
+													"|s",(gdle->post_fname)?gdle->post_fname->str:"",	/* filename rename */
+													"|s",(gdle->at_end_script)?gdle->at_end_script->str:"",	/* script to start at end */
 													links_str->str,
 													range_str->str,
 													ascan_str->str,
