@@ -25,11 +25,11 @@ typedef struct
 {
 	GString *nickname;
 	GString *remote_fname;
-	unsigned long int remote_fsize;
+	off_t remote_fsize;
 
 	GString *temp_local_fname;
-	unsigned long int range[2];	/* start position (included) and end position (excluded) in the file (valid when is_allocated==1) */
-	unsigned long int cur_dled;	/* number of bytes downloaded since the beginning of the range (valid when is_allocated==1) */
+	off_t range[2];	/* start position (included) and end position (excluded) in the file (valid when is_allocated==1) */
+	off_t cur_dled;	/* number of bytes downloaded since the beginning of the range (valid when is_allocated==1) */
 	
 	time_t last_start_time;
 	pthread_t thread_id;			/* if is_started==1, this is the thread_id of the download thread */
@@ -55,14 +55,23 @@ typedef struct
 {
 	unsigned int gdl_id;			/* it is a uniq ID identifying this GDL */
 	time_t start_time;			/* date of creation of this GDL */
-	unsigned long dl_offset;	/* to avoid erroneous speed computation, this var contains the number of */
-										/* still downloaded bytes (0 for create, -xxx for an attach */
+	off_t dl_offset;				/* to avoid erroneous speed computation, this var contains the number of */
+										/* already downloaded bytes (0 for create, -xxx for an attach */
 	int lock_fd;					/* it is the fd of the .lock file of this gdl */
 	unsigned int is_completed;	/* this flag is set to 0 until all parts of the file have been downloaded */
 										/* when ==1, all parts are here, gathering in progress (or soon start) */
 										/* if ==2, there is an internal error, this GDL is dead and will remain as is */
-	GString *local_fname;		/* it is the name of the local file after download */
-	unsigned long int total_size;
+	GString *local_fname;		/* it is the name of the local file after download (when fragments are gathered) */
+										/* After GDL creation, this variable cannot be modified */
+
+	GString *post_fname;			/* these 2 variables contains the final filename and the final directory, after fragment gathering */
+	GString *post_dir;			/* unlike local_fname, they can be altered at any time, even during fragment gathering */
+
+	GString *at_end_script;		/* these variable contains the name of a program (script or real program) to start at the end */
+										/* of GDL */
+
+	//unsigned long int total_size;
+	off_t total_size;
 										/* length of the file to download */
 	GPtrArray *gdl_links;		/* it is an array of all known nicks from where we (can) download */
 										/* each pointer points on a GDL_DL_ENTRY */
@@ -81,7 +90,7 @@ typedef struct
 typedef struct
 {
 	GString *temp_local_fname;		/* name of the temporary filename associated here */
-	unsigned long int range[2];	/* start position and end position in the file */
+	off_t range[2];					/* start position and end position in the file */
 } RANGE_ENTRY;
 
 /**********************************************************/
@@ -148,6 +157,26 @@ void do_gdl_as_add(unsigned int gdl_id, int filetype, char *pattern);
 /***********************************************/
 void do_gdl_as_del(unsigned int gdl_id, unsigned long gae_id);
 
+/****************************************************/
+/* change the final filename [and dirname] of a GDL */
+/*********************************************************************/
+/* if dirname == "", only filename is altered                        */
+/* if both filename and dirname == "", the file renaming is disabled */
+/*********************************************************************/
+void do_gdl_rename(unsigned int gdl_id, char *filename, char *dirname);
+
+/***************************************************************/
+/* change the name of the program to start at the end of a GDL */
+/***************************************************************/
+/* if filename is NULL or == "", the file renaming is disabled */
+/***************************************************************/
+void do_gdl_script(unsigned int gdl_id, char *programname);
+
+//************************************************************/
+/* call this function to update already existing socket TOS */
+/************************************************************/
+void gdl_alter_socket_tos(void);
+
 /* -------------------------------------------------------------------------- */
 /*************************************************************************/
 /* the following functions are used by the standard download function to */
