@@ -43,6 +43,7 @@
 #include "he3.h"
 #include "md.h"
 #include "macro.h"
+#include "sema.h"
 
 /* directory containing virtual shares */
 char *vshare_directory=NULL;
@@ -133,6 +134,7 @@ static int get_file_type_from_ext(char *fullname)
 						{"ps",4},
 						{"eps",4},
 						{"html",4},
+						{"htm",4},
 
 						/* exe type */
 						{"exe",5},
@@ -167,6 +169,7 @@ static int get_file_type_from_ext(char *fullname)
 						{"divx",7},
 						{"mov",7},
 						{"rm",7},
+						{"pva",7},	/* DVB card video format */
 
 						{NULL,-1}		/* last line */
 					};
@@ -1005,12 +1008,18 @@ static void send_a_db_result(int output_sck, char *dest_nick, DB_ENTRY *de, stru
 	GString *str;
 	GString *adapted;
 	int out;
+	int ttl_dl_slot;
+	int busy_slot;
+	int free_dl_slot;
 
 	str=g_string_sized_new(1024);
 
 	adapted=g_string_new(de->filename);
 	adapted=convert_path(adapted);
 
+	get_ul_slot_values(bl_semid,&ttl_dl_slot,&busy_slot);
+	free_dl_slot=ttl_dl_slot-busy_slot;
+
 	LOCK_READ(user_info);
 	if(md5sum==NULL)
 	{
