@@ -93,7 +93,7 @@
 /* output: 0=ok, !=0=error                                              */
 /*         on success, *cur_semid is the semaphore id to use with semop */
 /************************************************************************/
-int do_sema_init(char *keyfile, int *cur_semid, int spd_limit, int dl_spd_limit, int gath_spd_limit)
+int do_sema_init(char *keyfile, int *cur_semid, int spd_limit, int dl_spd_limit, int gath_spd_limit, int ttl_up_slot)
 {
 	int fd;
 	key_t key;
@@ -190,6 +190,17 @@ int do_sema_init(char *keyfile, int *cur_semid, int spd_limit, int dl_spd_limit,
 		v.val=gath_spd_limit;
 		if(semctl(semid,6,SETVAL,v)==-1)
 			perror("semctl6");
+
+		/* initialize upload slot control */
+		v.val=1;
+		if(semctl(semid,7,SETVAL,v)==-1)
+			perror("semctl7");
+		v.val=ttl_up_slot;
+		if(semctl(semid,8,SETVAL,v)==-1)
+			perror("semctl8");
+		v.val=0;								/* at the beginning, all upload slots are free */
+		if(semctl(semid,9,SETVAL,v)==-1)
+			perror("semctl9");
 	}
 	close(fd);
 	eofunc:
@@ -257,8 +268,8 @@ static void *sema_master(void *dm_val)
 /**********************************************/
 static void create_sema_master(int semid)
 {
-	pthread_attr_t thread_attr;
-	pthread_t thread_id;
+	static pthread_attr_t thread_attr;	/* these 2 variables must exist as long as the thread exist */
+	static pthread_t thread_id;
 
    pthread_attr_init (&thread_attr);
    pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
@@ -352,3 +363,214 @@ void get_gather_slices(int semid,int nb)
 	}
 }
 
+/******************************/
+/* lock the UL slot controler */
+/******************************/
+void lock_ul_slot_controler(int semid)
+{
+	struct sembuf get_ul_ctrl={UL_SLOT_SEMA,-1,SEM_UNDO};
+	/* lock the UL slot controler */
+	semop(semid,&get_ul_ctrl,1);
+}
+
+/*********************************/
+/* release the UL slot controler */
+/*********************************/
+void release_ul_slot_controler(int semid)
+{
+	struct sembuf release_ul_ctrl={UL_SLOT_SEMA,+1,SEM_UNDO};
+	/* release the UL slot controler */
+	semop(semid,&release_ul_ctrl,1);
+}
+
+/*********************************************************/
+/* get upload slot values (total ul slot and #busy slots */
+/*********************************************************/
+/* WARNING: this function locks the UL controler itself */
+/********************************************************/
+void get_ul_slot_values(int semid,int *ttl_slot, int *busy_slot)
+{
+	lock_ul_slot_controler(semid);
+
+	*ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
+	*busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+
+	release_ul_slot_controler(semid);
+}
+
+/*******************************************************************/
+/* count the number of free slots. This function performs NO LOCK  */
+/* and is only design to return the number of free slots at a time */
+/* If the number of free slot is negative, it is rounded to 0.     */
+/*******************************************************************/
+int number_of_free_slot(int semid)
+{
+	int ttl, busy;
+	int free_slt;
+
+	ttl=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
+	busy=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+
+	free_slt=ttl-busy;
+	if(free_slt<0)
+		free_slt=0;
+	return free_slt;
+}
+
+static int nb_local_ul=0;			/* number of upload slots used by this client */
+
+/******************************/
+/* try to get one upload slot */
+/************************************************/
+/* output: 0= fail to get slot, 1=slot obtained */
+/********************************************************/
+/* WARNING: this function locks the UL controler itself */
+/********************************************************/
+int try_to_get_ul_slot(int semid)
+{
+	int ttl_slot, busy_slot;
+	int ret=0;
+
+	lock_ul_slot_controler(semid);
+
+	ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
+	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+	if(busy_slot<ttl_slot)
+	{
+#if 0
+		union semun v;
+		ret=1;
+
+		v.val=busy_slot+1;
+		if(semctl(semid,UL_SLOT_BUSY_SEMA,SETVAL,v)==-1)
+		{
+			perror("try_to_get_ul_slot");
+			ret=0;
+		}
+#else
+		/* we must use this command because the previous code */
+		/* does not free the ul slot if the program ends */
+		struct sembuf get_one_ul_slot={UL_SLOT_BUSY_SEMA,+1,SEM_UNDO};
+
+		ret=1;
+		if(semop(semid,&get_one_ul_slot,1)==-1)
+		{
+			perror("try_to_get_ul_slot");
+			ret=0;
+		}
+#endif
+	}
+
+	if(ret)
+		nb_local_ul++;
+
+	release_ul_slot_controler(semid);
+
+	if(ret)
+	{	/* set the number of uploads of this client after releasing the ul_slot_controler */
+		SET_GSTATUS_UL(nb_local_ul);
+	}
+	return ret;
+}
+
+/************************************************/
+/* get an upload slot even if none is available */
+/********************************************************/
+/* WARNING: this function locks the UL controler itself */
+/********************************************************/
+void force_get_ul_slot(int semid)
+{
+#if 0
+	int busy_slot;
+	union semun v;
+	lock_ul_slot_controler(semid);
+	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+	v.val=busy_slot+1;
+	if(semctl(semid,UL_SLOT_BUSY_SEMA,SETVAL,v)==-1)
+	{
+		perror("force_get_ul_slot");
+	}
+#else
+	struct sembuf get_one_ul_slot={UL_SLOT_BUSY_SEMA,+1,SEM_UNDO};
+
+	lock_ul_slot_controler(semid);
+	if(semop(semid,&get_one_ul_slot,1)==-1)
+	{
+		perror("force_get_ul_slot");
+	}
+#endif
+	nb_local_ul++;
+
+	release_ul_slot_controler(semid);
+
+	/* set the number of uploads of this client after releasing the ul_slot_controler */
+	SET_GSTATUS_UL(nb_local_ul);
+}
+
+
+/*******************************************************/
+/* free an upload slot allocated by try_to_get_ul_slot */
+/*******************************************************/
+void free_one_ul_slot(int semid)
+{
+	int busy_slot;
+	lock_ul_slot_controler(semid);
+	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+	if(busy_slot>0)
+	{
+#if 0
+		union semun v;
+		v.val=busy_slot-1;
+		if(semctl(semid,UL_SLOT_BUSY_SEMA,SETVAL,v)==-1)
+		{
+			perror("free_one_ul_slot");
+		}
+#else
+		struct sembuf free_one_ul_slot_op={UL_SLOT_BUSY_SEMA,-1,SEM_UNDO};
+		if(semop(semid,&free_one_ul_slot_op,1)==-1)
+		{
+			perror("free_one_ul_slot");
+		}
+#endif
+		nb_local_ul--;
+	}
+	else
+	{
+		fprintf(stderr,"free_one_ul_slot: no slot is busy\n");
+	}
+	release_ul_slot_controler(semid);
+
+	/* set the number of uploads of this client after releasing the ul_slot_controler */
+	SET_GSTATUS_UL(nb_local_ul);
+}
+
+/*********************************/
+/* set the number of upload slot */
+/*********************************/
+void set_number_of_ul_slot(int semid, unsigned int number_of_slot)
+{
+	union semun v;
+	v.val=number_of_slot;
+
+	lock_ul_slot_controler(semid);
+	if(semctl(semid,UL_SLOT_TTL_SEMA,SETVAL,v)==-1)
+	{
+		perror("free_one_ul_slot");
+	}
+	release_ul_slot_controler(semid);
+}
+
+/*********************************/
+/* set the number of upload slot */
+/*********************************/
+int get_number_of_ul_slot(int semid)
+{
+	int ttl_slot;
+
+	lock_ul_slot_controler(semid);
+	ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
+	release_ul_slot_controler(semid);
+
+	return ttl_slot;
+}
+
