@@ -5,9 +5,10 @@
 
 #include <netinet/in.h>
 
-/* this structure is attached to each existing entry of the berkeley database */
+/* this structure describes one entry unode database */
 typedef struct
 {
+	char host_unresolved_addr[128];	/* unresolved hostname:port of this entry */
    time_t resolv_creation_date;     /* each resolved named has a date of solving allowing dynamic host IP to be refreshed */
    struct sockaddr_in dest_addr;    /* it is a ready to use address, including port and addresse type */
    unsigned char entry_enabled;     /* ==0, this entry is disabled, ==1, entry enabled */
